/*
 * This file is part of the coreboot project.
 *
 * Copyright (C) 2010 Anders Jenbo <anders@jenbo.dk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 */

#include <spd.h>
#include <delay.h>
#include <stdlib.h>
#include "vt694.h"
#include "raminit.h"

/*-----------------------------------------------------------------------------
Macros and definitions.
-----------------------------------------------------------------------------*/

/* Debugging macros. */
#if CONFIG_DEBUG_RAM_SETUP
#define PRINT_DEBUG(x)		print_debug(x)
#define PRINT_DEBUG_HEX8(x)	print_debug_hex8(x)
#define PRINT_DEBUG_HEX16(x)	print_debug_hex16(x)
#define PRINT_DEBUG_HEX32(x)	print_debug_hex32(x)


#define DUMPNORTH()		dump_pci_device(PCI_DEV(0, 0, 0))
#else
#define PRINT_DEBUG(x)
#define PRINT_DEBUG_HEX8(x)
#define PRINT_DEBUG_HEX16(x)
#define PRINT_DEBUG_HEX32(x)
#define DUMPNORTH()
#endif

#define NB PCI_DEV(0, 0, 0)

/* SDRAMC[2-0] - SDRAM Mode Select (SMS). Device 0 Offset 6C - SDRAM Control (00h) RW */
#define RAM_COMMAND_NORMAL	0x0
#define RAM_COMMAND_NOP		0x1
#define RAM_COMMAND_PRECHARGE	0x2
#define RAM_COMMAND_MRS		0x3
#define RAM_COMMAND_CBR		0x4

/*
Device 0 Offset 6A - Refresh Counter (00h) ...................RW
  7-0 Refresh Counter (in units of 16 MCLKs)
          00 DRAM Refresh Disabled.......................default
          01 32 MCLKs
          02 48 MCLKs
          03 64 MCLKs
          04 80 MCLKs
          05 96 MCLKs*/

static void sdram_set_registers(void)
{
	device_t north = (device_t) PCI_DEV(0, 0, 0);

	PRINT_DEBUG("Northbridge prior to SDRAM init:\n");
	DUMPNORTH();

	/* Set default NB options. */
	//1111 1110
	//1010 1100
	pci_write_config8(north, NBCR0, 0xac);
	//1101 1111
	//0000 0100
	pci_write_config8(north, NBCR1, 0x04);
	//1100 1000
	//0010 0000
	pci_write_config8(north, NBCR2, 0x20);
	//1001 1000
	//0000 0001
	pci_write_config8(north, NBCR3, 0x01);
	//0000 0100
	pci_write_config8(north, NBCR4, 0x04);

	/* Choose SDRAM. */
	pci_write_config8(north, DRAMTY, 0xFF);

	/* Set the DRBs to zero for now, this will be fixed later. */
	pci_write_config8(north, DRB0, 0x00);
	pci_write_config8(north, DRB1, 0x00);
	pci_write_config8(north, DRB2, 0x00);
	pci_write_config8(north, DRB3, 0x00);
	pci_write_config8(north, DRB4, 0x00);
	pci_write_config8(north, DRB5, 0x00);
	pci_write_config8(north, DRB6, 0x00);
	pci_write_config8(north, DRB7, 0x00);

	/* No, CMD 3T, Standard, CAS 3T, TRAS 6T, TRP 3T */
	pci_write_config8(north, DRAMT0, 0xe4);
	pci_write_config8(north, DRAMT1, 0xe4);
	pci_write_config8(north, DRAMT2, 0xe4);
	pci_write_config8(north, DRAMT3, 0xe4);

	
	/* Dram frequency select.
	 * enable 4K pages for 64M dram.
	 * Set to match FSB
	 */
	pci_write_config8(north, DRAMCL, 0x3c);

	/* Refresh counter
	 * Reset it
	 */
	pci_write_config8(north, RC, 0x00);

	/* DRAM read latch delay of 1.5 ns, MD drive 8 mA,
	 * high drive strength
	 */
	pci_write_config8(north, DRAMD, 0x7f);
}

static void sdram_set_spd_registers(void)
{
	PRINT_DEBUG("sdram_set_spd_registers:\n");
}

static void sdram_enable(void)
{
	PRINT_DEBUG("sdram_enable:\n");
	int i;

	/* 0. Wait until power/voltages and clocks are stable (200us). */
	udelay(200);

	/* 1. Apply NOP. Wait 200 clock cycles (200us should do). */
	PRINT_DEBUG("RAM Enable 1: Apply NOP\n");
	udelay(200);

	/* 2. Precharge all. Wait tRP. */
	PRINT_DEBUG("RAM Enable 2: Precharge all\n");
	udelay(1);

	/* 3. Perform 8 refresh cycles. Wait tRC each time. */
	PRINT_DEBUG("RAM Enable 3: CBR\n");
	for (i = 0; i < 8; i++) {
		udelay(1);
	}

	/* 4. Mode register set. Wait two memory cycles. */
	PRINT_DEBUG("RAM Enable 4: Mode register set\n");
	udelay(2);


	/* 6. Finally enable refresh. */
	PRINT_DEBUG("RAM Enable 6: Enable refresh\n");
	udelay(1);

	PRINT_DEBUG("Northbridge following SDRAM init:\n");
	DUMPNORTH();
}
