/*
 * This file is part of the coreboot project.
 *
 * Copyright (C) 2010 Anders Jenbo <anders@jenbo.dk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 */

/*
 * Datasheet:
 *   - Name: VIA VT82C694X Apollo Pro133A
 *   - PDF: ds_694x_ApolloPro133A.pdf
 */

/*
 * Host-to-PCI Bridge Registers.
 * The values in parenthesis are the default values as per datasheet.
 * Any addresses between 0x00 and 0xff not listed below are Reserved
 * and should not be touched.
 */


/* NBCR 0x50 - Configuration Registers
 * [7] In-Order Queue Depth (IOQD)
 *         1 = In-order queue = 4-level
 *         0 = In-order queue = 1-level
 * [6] Read-Around-Write
 *         0 = Disable
 *         1 = Enable
 * [5] Reserved
 * [4] Defer Retry When HLOCK Active
 *         0 = Disable
 *         1 = Enable
 * [1-3] Reserved
 * [0] CPU/PCI Master Read DRAM Timing
 *         0 = Start DRAM read after snoop complete
 *         1 = Start DRAM read befor snoop complete
 */
#define NBCR0   0x50

/* NBCR 0x51 - Configuration Registers
 * [7] CPU Read DRAM 0ws for Back-to-Back Read Transactions
 *         0 = Disable
 *         1 = Enable
 * [6] CPU Write DRAM 0ws for Back-to-Back Read Transactions
 *         0 = Disable
 *         1 = Enable
 * [5] Reserved
 * [4] Fast Response (HIT/HITM sample 1T erlier)
 *         0 = Disable
 *         1 = Enable
 * [3] Non-Posted IOW
 *         0 = Disable
 *         1 = Enable
 * [2] CE Silicon (Reserved): Zero Length Write
 *         0 = Disable
 *         1 = Enable - must be set to 1
 * [1] Reserved
 * [0] Concurrent PCI Master / Host Operation
 *         0 = Disable
 *         1 = Enable
 */
#define NBCR1   0x51

/* NBCR 0x52 - Configuration Registers
 * [7] GTL I/O Buffer Pullup
 *         0 = Disable (if MAB6 Strap is 1)
 *         1 = Enable (if MAB6 Strap is 0)
 * [6] RAW Write Retire Policy (After 2 Writes)
 *         0 = Disable
 *         1 = Enable
 * [5] Quick Start Selected
 *         0 = Disable (if MAB10 Strap is 1)
 *         1 = Enable (if MAB10 Strap is 0)
 * [0-4] Snoop Stall Count
 *         0x00 = Disable dynamic defer
 *         0x01-0x1f (default is 0x10)
 */
#define NBCR2   0x52

/* NBCR 0x53 - Configuration Registers
 * [7] HREQ
 *         0 = Disable
 *         1 = Enable
 * [6] SDRAM Frequincy Higher Then CPU FSB Frequincy
 *         0 = Disable
 *         1 = Enable
 * [5] PCI/AGP Master-to-CPU / CPU-to-AGP Slave Concurrency
 *         0 = Disable
 *         1 = Enable
 * [4] HPRI Function
 *         0 = Disable
 *         1 = Enable
 * [3] P6Lock Function
 *         0 = Disable
 *         1 = Enable
 * [2] Line Write / Write Back Without Lmplicit Write Back Data
 *         0 = Disable
 *         1 = Enable
 * [1] PCI Master Pipeline Access
 *         0 = Disable
 *         1 = Enable
 * [0] Initialization of Fast Write Address Selection
 *         1 = Head
 *         0 = Tail
 */
#define NBCR3   0x53

/* NBCR 0x54 - Configuration Registers
 * [7-6] Reserved (Do Not Programm)
 * [3-5] Reserved
 * [2] CE Silicon (Reserved): Zero Length Write
 *         0 = Disable
 *         1 = Enable - must be set to 1
 * [1] Invalid CPU Internal Cache on PCI Master Access
 *         0 = Disable
 *         1 = Enable
 * [0] 1-1-1-1 PMRDY for PCI Master Access
 *         0 = Disable
 *         1 = Enable
 */
#define NBCR4   0x54

/* DRAMC - DRAM type
 * 0x60
 *
 * 00 FPG
 * 01 EDO
 * 10 Reserved
 * 11 SDRAM
 *
 * [7:6] Bank 6/7 type
 * [5:4] Bank 4/5 type
 * [3:2] Bank 2/3 type
 * [1:0] Bank 1/0 type
 */
#define DRAMTY  0x60


/* DRB[0:7] - DRAM Row Boundary Registers
 * 0x60 - 0x67
 *
 * An array of 8 byte registers, which hold the ending memory address
 * assigned to each bank, in 8MB granularity.
 *
 * 0x5a DRB0 = Total memory in row0 (in 8 MB)
 * 0x5b DRB1 = Total memory in row0+1 (in 8 MB)
 * 0x5c DRB2 = Total memory in row0+1+2 (in 8 MB)
 * 0x5d DRB3 = Total memory in row0+1+2+3 (in 8 MB)
 * 0x5e DRB4 = Total memory in row0+1+2+3+4 (in 8 MB)
 * 0x5f DRB5 = Total memory in row0+1+2+3+4+5 (in 8 MB)
 * 0x56 DRB6 = Total memory in row0+1+2+3+4+5+6 (in 8 MB)
 * 0x57 DRB7 = Total memory in row0+1+2+3+4+5+6+7 (in 8 MB)
 */
#define DRB0    0x5a
#define DRB1    0x5b
#define DRB2    0x5c
#define DRB3    0x5d
#define DRB4    0x5e
#define DRB5    0x5f
#define DRB6    0x56
#define DRB7    0x57


/* SDRAM Timing for all banks
 * no interleave
 * Standard DIMM type
 * ras precharge 3T, RAS pulse 3T
 * cas2 is 0xd6, cas3 is 0xe6
 */
#define DRAMT0  0x64
#define DRAMT1  0x65
#define DRAMT2  0x66
#define DRAMT3  0x67


#define DRAMC   0x68 /* DRAM Control. */
#define DRAMCL  0x69 /* DRAM Clock. */
#define RC      0x6a /* Refresh counter. */
#define DRAMD   0x6d /* DRAM Drive Strength. */
#define DRAMECC 0x6e /* DRAM ECC. */
#define FDHC    0x63 /* Fixed SDRAM Hole Control. */
